"""Search module with saved searches and suggestions."""

from hippocampai.search.saved_searches import SavedSearchManager
from hippocampai.search.suggestions import SearchSuggestionEngine

__all__ = ["SavedSearchManager", "SearchSuggestionEngine"]
